#include "pico/stdlib.h"
#include <stdio.h>
#include <math.h>

#include "pico/time.h"
#include "hardware/irq.h"
#include "hardware/structs/adc.h"
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_pwm\include\hardware\pwm.h>
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_gpio\include\hardware\gpio.h>
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_sync\include\hardware\sync.h>
#include "hardware/resets.h"
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_spi\include\hardware\spi.h>
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_spi\spi.c>

#define PWM_PIN1 2    // pin 4
#define PWM_PIN2 3    // pin 5
#define GPIO4 4     // pin 6    

 #define clkDiv 625  // 125000000/625=200000 interrupts/sec

/*
   GPIO 16 (pin 21) MISO
   GPIO 17 (pin 22) CS
   GPIO 18 (pin 24) SCK
   GPIO 19 (pin 25) MOSI
   GPIO 20 (pin 26) DC
   3.3v (pin 36) -> VCC
   GND (pin 38)  -> GND
*/

#define TFT_MISO 16 
#define TFT_CS 17 
#define TFT_SCK 18
#define TFT_MOSI 19 
#define TFT_DC 20 

//-----------------------------------------------------------------------------------

static void approxSleepMs(int n){
  for(int k=0 ; k<n ; k++){
    for( int kk=0 ; kk<17850 ; kk++){
      asm volatile("nop");
      }
    }
  }
  
//---------------------------------------------------------------

double fSample ;

void abort(){
  printf("abort..\n") ;
  printf("\n") ;
  while(1){} ;
  }

#define spiFIFOsize 256
#define spiFIFOmask (spiFIFOsize-1)

int spiFIFO[spiFIFOsize] ;
int spiFIFOinPtr ;
int spiFIFOoutPtr ;

void spiFIFOinit(){
  spiFIFOinPtr=0 ;
  spiFIFOoutPtr=1 ;
  }

uint16_t spiBuf16[2];
volatile int spiData ;

int spiFIFOfull(){
  int state ;
  int flag ;
  state=save_and_disable_interrupts () ;
  flag= spiFIFOinPtr==spiFIFOoutPtr ;
  restore_interrupts (state) ;
  return flag ;
  }

void PWMirq() {
  gpio_put(GPIO4, 1);
  spiBuf16[0] =spiData ;
  pwm_clear_irq(pwm_gpio_to_slice_num(PWM_PIN1));
  spi_write16_blocking(spi_default, spiBuf16, 1);
  spiData=spiFIFO[spiFIFOoutPtr] ;
  spiFIFOoutPtr=(spiFIFOoutPtr+1) & spiFIFOmask ;
  gpio_put(GPIO4, 0);
  }

void PWMsetup(){    
  gpio_set_function(PWM_PIN1, GPIO_FUNC_PWM);
  gpio_set_function(PWM_PIN2, GPIO_FUNC_PWM);
   // Figure out which slice we just connected to the pin
  uint slice_num1 = pwm_gpio_to_slice_num(PWM_PIN1);
  uint slice_num2 = pwm_gpio_to_slice_num(PWM_PIN2);
  // Mask our slice's IRQ output into the PWM block's single interrupt line,
  // and register our interrupt handler
  pwm_clear_irq(slice_num1);
  pwm_set_irq_enabled(slice_num1, true);
  irq_set_exclusive_handler(PWM_IRQ_WRAP, PWMirq);

  irq_set_enabled(PWM_IRQ_WRAP, true);

  // Get some sensible defaults for the slice configuration. 
  pwm_config config1 = pwm_get_default_config();
  // Set divider, reduces counter clock to sysclock/this value
  pwm_config_set_clkdiv(&config1, 1.f);
  pwm_config_set_output_polarity (&config1,1,0) ; // invert A channel

  // Load the configuration into our PWM slice, and set it running.
  pwm_init(slice_num1, &config1, true);
  pwm_set_wrap(slice_num1,(clkDiv-1) );
  fSample=125e6/clkDiv/1.0 ;
  printf("fSample=%15.5f\n",fSample) ;
  printf("PWM_PIN1=%d\n",PWM_PIN1) ;
  printf("PWM_PIN2=%d\n",PWM_PIN2) ;
  }

int main() {
  stdio_init_all();
  gpio_init(GPIO4);
  gpio_set_dir(GPIO4, GPIO_OUT);
   
  busy_wait_us_32(200000) ;
  printf("//\n") ;
  printf("//\n") ;
  printf("Hello IRQloadMeasurementV01 !\n") ;

  spiFIFOinit() ;

  gpio_init(TFT_CS);
  gpio_set_dir(TFT_CS, GPIO_OUT);
  gpio_put(TFT_CS, 1);
  spi_init(spi_default,  8000000);
  // gpio_set_function(PICO_DEFAULT_SPI_RX_PIN, GPIO_FUNC_SPI);
  gpio_set_function(PICO_DEFAULT_SPI_SCK_PIN, GPIO_FUNC_SPI);
  gpio_set_function(PICO_DEFAULT_SPI_TX_PIN, GPIO_FUNC_SPI);
  spi_set_format(spi_default,24,0,0,SPI_MSB_FIRST) ;

  PWMsetup() ;
  pwm_set_gpio_level(PWM_PIN1, 500);
  pwm_set_gpio_level(PWM_PIN2, 400);
  
  int k=0 ;
  printf("measure with Interrupts:\n") ;
  for(k=0 ; k<5 ; k++){
    long long t1=time_us_64() ;
    approxSleepMs(1);
    long long t2=time_us_64() ;
    int dt=(int) (t2-t1) ;
    printf("t1=%lli t2=%lli t2-t1=%d\n",t1,t2,dt) ;
    busy_wait_us_32(500000) ;
    }
  printf("\nDisable interrupts...\n") ;
  int state=save_and_disable_interrupts () ;
  for(k=0 ; k<5 ; k++){
    long long t1=time_us_64() ;
    approxSleepMs(1);
    long long t2=time_us_64() ;
    int dt=(int) (t2-t1) ;
    printf("t1=%lli t2=%lli t2-t1=%d\n",t1,t2,dt) ;
    busy_wait_us_32(500000) ;
    }
  }

 
  

